#include <stdlib.h>
#include <stdio.h>
#include <windows.h>

typedef INT (*METADATA_PROC)();
typedef INT (*GRAB_FRAME_PROC)(BYTE *,INT);

__declspec( dllimport) int getWidth();
__declspec( dllimport) int getHeight();
__declspec( dllimport) int getFrameSize();
__declspec( dllimport) int getFrame(BYTE * pBuf, int lBuf);

HINSTANCE fgHandle = NULL;
METADATA_PROC fgGetWidth;
METADATA_PROC fgGetHeight;
METADATA_PROC fgGetFrameSize;
GRAB_FRAME_PROC fgGetFrame;



static void clearProcAddresses()
{
	fgGetWidth = NULL;
	fgGetHeight = NULL;
	fgGetFrameSize = NULL;
	fgGetFrame = NULL;	
}




static int loadDll()
{
	clearProcAddresses();
	
	fgHandle = LoadLibrary("fgClient");
	if (!fgHandle) {
		return 0;
	}

	fgGetWidth = (METADATA_PROC) GetProcAddress (fgHandle, "getWidth");
	fgGetHeight = (METADATA_PROC) GetProcAddress (fgHandle, "getHeight");
	fgGetFrameSize = (METADATA_PROC) GetProcAddress (fgHandle, "getFrameSize");
	fgGetFrame = (GRAB_FRAME_PROC) GetProcAddress (fgHandle, "getFrame");

	if (!fgGetWidth || !fgGetHeight || !fgGetFrameSize || !fgGetFrame) {
		FreeLibrary (fgHandle);
		fgHandle = NULL;
		return 0;
	}

	return 1;
}



int framecap_init()
{
	return loadDll();
}



int framecap_shutdown()
{
	if (fgHandle) {
		FreeLibrary (fgHandle);
		fgHandle = NULL;
	}
	return 1; 
}



int framecap_getHeight()
{
	return fgGetHeight();
}



int framecap_getWidth()
{
	return fgGetWidth();
}



int framecap_getFrameSize()
{
	return fgGetFrameSize();
}



/* slightly antiaesthetic ... */
void framecap_frameAsBitmapArray (char *dest, char const *src, int height, int width)
{
	int size = height * width;
	int y;

	dest += size * 4;
	for (y = 0; y < height; ++y) {
		int x;
		char *row;

		dest -= width * 4;
		row = dest;

		for (x = 0; x < width; ++x) {
			*(row++) = 255;
			*(row+2) = *(src++);
			*(row+1) = *(src++);
			*(row+0) = *(src++);
			row += 3;
		}
	}
}



int framecap_getFrame (char *dest, int size)
{
	return (fgGetFrame)(dest, size);
}

